# -*- coding: utf-8 -*-
"""
/***************************************************************************
 BiotopManager
                                 A QGIS plugin
 Dieses Plugin verwaltet Biotope
 Generated by Plugin Builder: http://g-sherman.github.io/Qgis-Plugin-Builder/
                              -------------------
        begin                : 2018-06-27
        git sha              : $Format:%H$
        copyright            : (C) 2018 by GBD GmbH
        email                : gebbert@gbd-consult.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""

from PyQt5 import Qt, QtSvg


def convert_html_to_pdf(html: str, filename: str) -> bool:
    """Convert a HTML string into a PDF file

    Args:
        html: A string that contains the HTML content
        filename: The PDF file

    Returns:
        True if the conversion succeeded

    >>> from qgis.core import QgsApplication
    >>> app = QgsApplication([], False)
    >>> app.initQgis()
    >>> html = "<h1>Hello, World!</h1>"
    >>> filename = "test_html.pdf"
    >>> convert_html_to_pdf(html=html, filename=filename)
    True

    """

    printer = Qt.QPrinter(Qt.QPrinter.PrinterResolution)
    printer.setOutputFormat(Qt.QPrinter.PdfFormat)
    printer.setPaperSize(Qt.QPrinter.A4)
    printer.setOutputFileName(filename)

    doc = Qt.QTextDocument()
    doc.setHtml(html)
    # doc.setPageSize(Qt.QPrinter.A4)
    # doc.setPageSize(Qt.QSizeF(printer.pageRect().size()))
    doc.print(printer)

    return True


def convert_svg_to_pdf(svg_filename: str, filename: str) -> bool:
    """Convert a HTML string into a PDF file

    Args:
        html: A string that contains the HTML content
        filename: The PDF file

    Returns:
        True if the conversion succeeded

    >>> import os
    >>> from qgis.core import QgsApplication
    >>> from biotopmanager.common import gui
    >>> app = QgsApplication([], False)
    >>> app.initQgis()
    >>> svg_filename = os.path.join(os.path.dirname(gui.__file__), "Biotopformular.svg")
    >>> filename = "test_svg.pdf"
    >>> convert_svg_to_pdf(svg_filename=svg_filename, filename=filename)
    True

    """
    renderer = QtSvg.QSvgRenderer()
    renderer.load(svg_filename)

    printer = Qt.QPrinter(Qt.QPrinter.PrinterResolution)
    printer.setOutputFormat(Qt.QPrinter.PdfFormat)
    printer.setPaperSize(Qt.QPrinter.A4)
    printer.setResolution(600)
    printer.setOutputFileName(filename)

    painter = Qt.QPainter()
    if not painter.begin(printer):
        raise Exception("Unable to call print")
    renderer.render(painter)
    painter.end()

    return True


if __name__ == "__main__":
    import doctest

    doctest.testmod()
